/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.view.fx;

import static javafx.scene.paint.Color.BISQUE;
import static javafx.scene.paint.Color.LIGHTGREY;
import static javafx.scene.paint.Color.LIGHTSEAGREEN;
import static javafx.scene.paint.Color.WHITE;
import static org.eclipse.swt.widgets.Display.getCurrent;

import org.eclipse.swt.graphics.Device;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.annotation.valueprovider.IAnnotationValueProvider;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;

import javafx.scene.paint.Color;

/**
 * Utils for the FX implementation of the annotations view.
 * 
 * @author bayha
 */
public class AnnotationsFXUtils {

	/** Background color for table cells, which are not editable and empty. */
	public static final Color READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR = LIGHTGREY;

	/** SWT background color for table cells, which are not editable and empty. */
	@Deprecated
	public static final org.eclipse.swt.graphics.Color READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR_SWT =
			convertFXColortoSWT(READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR);

	/** Background color for table cells, which are not editable. */
	public static final Color READ_ONLY_ANNOTATION_BACKGROUND_COLOR = BISQUE;

	/** Background color for table cells of the selected element. */
	protected static final Color SELECTED_ANNOTATION_BACKGROUND_COLOR = LIGHTSEAGREEN;

	/**
	 * @param annotationEntry
	 * @param annotationType
	 * @return
	 */
	public static Color getBackgroundColorForEntry(AnnotationEntry annotationEntry,
			Class<? extends IAnnotatedSpecification> annotationType) {
		IAnnotationValueProvider<IAnnotatedSpecification> valueProvider =
				annotationEntry.getAnnotationValueProvider(annotationType);
		IAnnotatedSpecification specification = annotationEntry.getSpecification(annotationType);

		if(valueProvider != null && valueProvider.getAnnotationValue(specification) != null &&
				!valueProvider.canEdit(specification)) {
			return READ_ONLY_ANNOTATION_BACKGROUND_COLOR;
		}

		if(valueProvider == null || !valueProvider.canEdit(specification)) {
			return READ_ONLY_EMPTY_ANNOTATION_BACKGROUND_COLOR;
		}

		return WHITE;
	}

	/**
	 * @param annotationEntry
	 * @param annotationType
	 * @param defaultColor
	 * @return
	 */
	@Deprecated
	public static org.eclipse.swt.graphics.Color getSWTBackgroundColorForEntry(
			AnnotationEntry annotationEntry,
			Class<? extends IAnnotatedSpecification> annotationType,
			org.eclipse.swt.graphics.Color defaultColor) {
		Color bgCol = getBackgroundColorForEntry(annotationEntry, annotationType);

		if(bgCol.equals(WHITE)) {
			return defaultColor;
		}

		return convertFXColortoSWT(bgCol);
	}

	/**
	 * @param bgCol
	 * @return
	 */
	private static org.eclipse.swt.graphics.Color convertFXColortoSWT(Color bgCol) {
		Device device = getCurrent();
		int red = (int)bgCol.getRed();
		int green = (int)bgCol.getGreen();
		int blue = (int)bgCol.getBlue();
		return new org.eclipse.swt.graphics.Color(device, red, green, blue);
	}
}
